@@ -1,4 +1,4 @@
-// Copyright 2019 Red Hat, Inc
+// Copyright 2020 Red Hat, Inc
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -12,7 +12,7 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.)
 
-package v0_3
+package v0_4
 
 import (
 	"fmt"
@@ -30,7 +30,7 @@ import (
 	"github.com/coreos/butane/translate"
 
 	"github.com/coreos/ignition/v2/config/util"
-	"github.com/coreos/ignition/v2/config/v3_2/types"
+	"github.com/coreos/ignition/v2/config/v3_3/types"
 	"github.com/coreos/vcontext/path"
 	"github.com/coreos/vcontext/report"
 	"github.com/stretchr/testify/assert"
@@ -45,7 +45,7 @@ var (
 
 func init() {
 	if runtime.GOOS == "windows" {
-		osStatName = "CreateFile"
+		osStatName = "GetFileAttributesEx"
 		osNotFound = "The system cannot find the file specified."
 	} else {
 		osStatName = "stat"
@@ -682,7 +682,7 @@ func TestTranslateLink(t *testing.T) {
 					Name: util.StrToPtr("bazquux"),
 				},
 				Overwrite: util.BoolToPtr(true),
-				Target:    "/bar",
+				Target:    util.StrToPtr("/bar"),
 				Hard:      util.BoolToPtr(false),
 			},
 			types.Link{
@@ -699,7 +699,7 @@ func TestTranslateLink(t *testing.T) {
 					Overwrite: util.BoolToPtr(true),
 				},
 				LinkEmbedded1: types.LinkEmbedded1{
-					Target: "/bar",
+					Target: util.StrToPtr("/bar"),
 					Hard:   util.BoolToPtr(false),
 				},
 			},
@@ -765,7 +765,7 @@ func TestTranslateFilesystem(t *testing.T) {
 				},
 			}
 			expected := []types.Filesystem{test.out}
-			actual, translations, r := in.ToIgn3_2Unvalidated(common.TranslateOptions{})
+			actual, translations, r := in.ToIgn3_3Unvalidated(common.TranslateOptions{})
 			r = confutil.TranslateReportPaths(r, translations)
 			baseutil.VerifyReport(t, test.in, r)
 			assert.Equal(t, expected, actual.Storage.Filesystems, "translation mismatch")
@@ -816,7 +816,7 @@ func TestTranslateMountUnit(t *testing.T) {
 			},
 			types.Config{
 				Ignition: types.Ignition{
-					Version: "3.2.0",
+					Version: "3.3.0",
 				},
 				Storage: types.Storage{
 					Filesystems: []types.Filesystem{
@@ -868,7 +868,7 @@ RequiredBy=local-fs.target`),
 						{
 							Name:   "foo-bar",
 							Device: util.StrToPtr("/dev/bar"),
-							Clevis: &Clevis{
+							Clevis: Clevis{
 								Tang: []Tang{
 									{
 										URL: "http://example.com",
@@ -881,7 +881,7 @@ RequiredBy=local-fs.target`),
 			},
 			types.Config{
 				Ignition: types.Ignition{
-					Version: "3.2.0",
+					Version: "3.3.0",
 				},
 				Storage: types.Storage{
 					Filesystems: []types.Filesystem{
@@ -896,7 +896,7 @@ RequiredBy=local-fs.target`),
 						{
 							Name:   "foo-bar",
 							Device: util.StrToPtr("/dev/bar"),
-							Clevis: &types.Clevis{
+							Clevis: types.Clevis{
 								Tang: []types.Tang{
 									{
 										URL: "http://example.com",
@@ -945,7 +945,7 @@ RequiredBy=remote-fs.target`),
 			},
 			types.Config{
 				Ignition: types.Ignition{
-					Version: "3.2.0",
+					Version: "3.3.0",
 				},
 				Storage: types.Storage{
 					Filesystems: []types.Filesystem{
@@ -994,7 +994,7 @@ RequiredBy=local-fs.target`),
 						{
 							Name:   "foo-bar",
 							Device: util.StrToPtr("/dev/bar"),
-							Clevis: &Clevis{
+							Clevis: Clevis{
 								Tang: []Tang{
 									{
 										URL: "http://example.com",
@@ -1007,7 +1007,7 @@ RequiredBy=local-fs.target`),
 			},
 			types.Config{
 				Ignition: types.Ignition{
-					Version: "3.2.0",
+					Version: "3.3.0",
 				},
 				Storage: types.Storage{
 					Filesystems: []types.Filesystem{
@@ -1021,7 +1021,7 @@ RequiredBy=local-fs.target`),
 						{
 							Name:   "foo-bar",
 							Device: util.StrToPtr("/dev/bar"),
-							Clevis: &types.Clevis{
+							Clevis: types.Clevis{
 								Tang: []types.Tang{
 									{
 										URL: "http://example.com",
@@ -1078,7 +1078,7 @@ RequiredBy=remote-fs.target`),
 			},
 			types.Config{
 				Ignition: types.Ignition{
-					Version: "3.2.0",
+					Version: "3.3.0",
 				},
 				Storage: types.Storage{
 					Filesystems: []types.Filesystem{
@@ -1100,11 +1100,96 @@ RequiredBy=remote-fs.target`),
 				},
 			},
 		},
+		// swap, no options
+		{
+			Config{
+				Storage: Storage{
+					Filesystems: []Filesystem{
+						{
+							Device:        "/dev/disk/by-label/foo",
+							Format:        util.StrToPtr("swap"),
+							WithMountUnit: util.BoolToPtr(true),
+						},
+					},
+				},
+			},
+			types.Config{
+				Ignition: types.Ignition{
+					Version: "3.3.0",
+				},
+				Storage: types.Storage{
+					Filesystems: []types.Filesystem{
+						{
+							Device: "/dev/disk/by-label/foo",
+							Format: util.StrToPtr("swap"),
+						},
+					},
+				},
+				Systemd: types.Systemd{
+					Units: []types.Unit{
+						{
+							Enabled: util.BoolToPtr(true),
+							Contents: util.StrToPtr(`# Generated by Butane
+[Swap]
+What=/dev/disk/by-label/foo
+
+[Install]
+RequiredBy=swap.target`),
+							Name: "dev-disk-by\\x2dlabel-foo.swap",
+						},
+					},
+				},
+			},
+		},
+		// swap with options
+		{
+			Config{
+				Storage: Storage{
+					Filesystems: []Filesystem{
+						{
+							Device:        "/dev/disk/by-label/foo",
+							Format:        util.StrToPtr("swap"),
+							MountOptions:  []string{"pri=1", "discard=pages"},
+							WithMountUnit: util.BoolToPtr(true),
+						},
+					},
+				},
+			},
+			types.Config{
+				Ignition: types.Ignition{
+					Version: "3.3.0",
+				},
+				Storage: types.Storage{
+					Filesystems: []types.Filesystem{
+						{
+							Device:       "/dev/disk/by-label/foo",
+							Format:       util.StrToPtr("swap"),
+							MountOptions: []types.MountOption{"pri=1", "discard=pages"},
+						},
+					},
+				},
+				Systemd: types.Systemd{
+					Units: []types.Unit{
+						{
+							Enabled: util.BoolToPtr(true),
+							Contents: util.StrToPtr(`# Generated by Butane
+[Swap]
+What=/dev/disk/by-label/foo
+Options=pri=1,discard=pages
+
+[Install]
+RequiredBy=swap.target`),
+							Name: "dev-disk-by\\x2dlabel-foo.swap",
+						},
+					},
+				},
+			},
+		},
 	}
 
 	for i, test := range tests {
 		t.Run(fmt.Sprintf("translate %d", i), func(t *testing.T) {
-			out, translations, r := test.in.ToIgn3_2Unvalidated(common.TranslateOptions{})
+			out, translations, r := test.in.ToIgn3_3Unvalidated(common.TranslateOptions{})
 			r = confutil.TranslateReportPaths(r, translations)
 			baseutil.VerifyReport(t, test.in, r)
 			assert.Equal(t, test.out, out, "bad output")
@@ -1291,7 +1376,7 @@ func TestTranslateTree(t *testing.T) {
 						},
 					},
 					LinkEmbedded1: types.LinkEmbedded1{
-						Target: "../file",
+						Target: util.StrToPtr("../file"),
 					},
 				},
 				{
@@ -1299,7 +1384,7 @@ func TestTranslateTree(t *testing.T) {
 						Path: "/subdir/bad-link",
 					},
 					LinkEmbedded1: types.LinkEmbedded1{
-						Target: "../nonexistent",
+						Target: util.StrToPtr("../nonexistent"),
 					},
 				},
 				{
@@ -1307,7 +1392,7 @@ func TestTranslateTree(t *testing.T) {
 						Path: "/subdir/link",
 					},
 					LinkEmbedded1: types.LinkEmbedded1{
-						Target: "../file",
+						Target: util.StrToPtr("../file"),
 					},
 				},
 			},
@@ -1361,7 +1446,7 @@ func TestTranslateTree(t *testing.T) {
 						Path: "/subdir/link",
 					},
 					LinkEmbedded1: types.LinkEmbedded1{
-						Target: "../file",
+						Target: util.StrToPtr("../file"),
 					},
 				},
 			},
@@ -1457,7 +1542,7 @@ func TestTranslateTree(t *testing.T) {
 			inLinks: []Link{
 				{
 					Path:   "/link",
-					Target: "file",
+					Target: util.StrToPtr("file"),
 				},
 				{
 					Path: "/link-partial",
@@ -1631,7 +1716,7 @@ func TestTranslateTree(t *testing.T) {
 			if test.options != nil {
 				options = *test.options
 			}
-			actual, translations, r := config.ToIgn3_2Unvalidated(options)
+			actual, translations, r := config.ToIgn3_3Unvalidated(options)
 
 			r = confutil.TranslateReportPaths(r, translations)
 			baseutil.VerifyReport(t, config, r)
@@ -1659,7 +1744,7 @@ func TestTranslateIgnition(t *testing.T) {
 		{
 			Ignition{},
 			types.Ignition{
-				Version: "3.2.0",
+				Version: "3.3.0",
 			},
 		},
 		{
@@ -1676,7 +1761,7 @@ func TestTranslateIgnition(t *testing.T) {
 				},
 			},
 			types.Ignition{
-				Version: "3.2.0",
+				Version: "3.3.0",
 				Config: types.IgnitionConfig{
 					Merge: []types.Resource{
 						{
@@ -1699,7 +1784,7 @@ func TestTranslateIgnition(t *testing.T) {
 				},
 			},
 			types.Ignition{
-				Version: "3.2.0",
+				Version: "3.3.0",
 				Proxy: types.Proxy{
 					HTTPProxy: util.StrToPtr("https://example.com:8080"),
 					NoProxy:   []types.NoProxyItem{types.NoProxyItem("example.com")},
@@ -1719,7 +1804,7 @@ func TestTranslateIgnition(t *testing.T) {
 				},
 			},
 			types.Ignition{
-				Version: "3.2.0",
+				Version: "3.3.0",
 				Security: types.Security{
 					TLS: types.TLS{
 						CertificateAuthorities: []types.Resource{
@@ -1749,25 +1834,72 @@ func TestTranslateIgnition(t *testing.T) {
 	}
 }
 
-// TestToIgn3_2 tests the config.ToIgn3_2 function ensuring it will generate a valid config even when empty. Not much else is
-// tested since it uses the Ignition translation code which has its own set of tests.
-func TestToIgn3_2(t *testing.T) {
+// TestTranslateKernelArguments tests translating the butane kernel_arguments.{should_exist,should_not_exist}.[i] entries to
+// ignition kernelArguments.{shouldExist,shouldNotExist}.[i] entries.
+//
+// KernelArguments do not use a custom translation function (it utilizes the MergeP2 functionality) so pass an entire config
+func TestTranslateKernelArguments(t *testing.T) {
 	tests := []struct {
 		in  Config
 		out types.Config
 	}{
 		{
-			Config{},
+			Config{
+				KernelArguments: KernelArguments{
+					ShouldExist: []KernelArgument{
+						"foo",
+					},
+					ShouldNotExist: []KernelArgument{
+						"bar",
+					},
+				},
+			},
 			types.Config{
 				Ignition: types.Ignition{
-					Version: "3.2.0",
+					Version: "3.3.0",
+				},
+				KernelArguments: types.KernelArguments{
+					ShouldExist: []types.KernelArgument{
+						"foo",
+					},
+					ShouldNotExist: []types.KernelArgument{
+						"bar",
+					},
 				},
 			},
 		},
 	}
 	for i, test := range tests {
 		t.Run(fmt.Sprintf("translate %d", i), func(t *testing.T) {
-			actual, translations, r := test.in.ToIgn3_2Unvalidated(common.TranslateOptions{})
+			actual, translations, r := test.in.ToIgn3_3Unvalidated(common.TranslateOptions{})
+			r = confutil.TranslateReportPaths(r, translations)
+			baseutil.VerifyReport(t, test.in, r)
+			assert.Equal(t, test.out, actual, "translation mismatch")
+			assert.Equal(t, report.Report{}, r, "non-empty report")
+			assert.NoError(t, translations.DebugVerifyCoverage(actual), "incomplete TranslationSet coverage")
+		})
+	}
+}
+
+// TestToIgn3_3 tests the config.ToIgn3_3 function ensuring it will generate a valid config even when empty. Not much else is
+// tested since it uses the Ignition translation code which has its own set of tests.
+func TestToIgn3_3(t *testing.T) {
+	tests := []struct {
+		in  Config
+		out types.Config
+	}{
+		{
+			Config{},
+			types.Config{
+				Ignition: types.Ignition{
+					Version: "3.3.0",
+				},
+			},
+		},
+	}
+	for i, test := range tests {
+		t.Run(fmt.Sprintf("translate %d", i), func(t *testing.T) {
+			actual, translations, r := test.in.ToIgn3_3Unvalidated(common.TranslateOptions{})
 			r = confutil.TranslateReportPaths(r, translations)
 			baseutil.VerifyReport(t, test.in, r)
 			assert.Equal(t, test.out, actual, "translation mismatch")
