@@ -12,20 +12,20 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.)
 
-package v1_3
+package v1_5
 
 import (
 	"fmt"
 	"testing"
 
 	baseutil "github.com/coreos/butane/base/util"
-	base "github.com/coreos/butane/base/v0_3"
+	base "github.com/coreos/butane/base/v0_5"
 	"github.com/coreos/butane/config/common"
 	confutil "github.com/coreos/butane/config/util"
 	"github.com/coreos/butane/translate"
 
 	"github.com/coreos/ignition/v2/config/util"
-	"github.com/coreos/ignition/v2/config/v3_2/types"
+	"github.com/coreos/ignition/v2/config/v3_4/types"
 	"github.com/coreos/vcontext/path"
 	"github.com/coreos/vcontext/report"
 	"github.com/stretchr/testify/assert"
@@ -46,7 +46,7 @@ func TestTranslateBootDevice(t *testing.T) {
 			Config{},
 			types.Config{
 				Ignition: types.Ignition{
-					Version: "3.2.0",
+					Version: "3.4.0",
 				},
 			},
 			[]translate.Translation{
@@ -89,7 +89,7 @@ func TestTranslateBootDevice(t *testing.T) {
 			},
 			types.Config{
 				Ignition: types.Ignition{
-					Version: "3.2.0",
+					Version: "3.4.0",
 				},
 				Storage: types.Storage{
 					Disks: []types.Disk{
@@ -152,6 +152,7 @@ func TestTranslateBootDevice(t *testing.T) {
 			Config{
 				BootDevice: BootDevice{
 					Luks: BootDeviceLuks{
+						Discard: util.BoolToPtr(true),
 						Tang: []base.Tang{{
 							URL:        "https://example.com/",
 							Thumbprint: util.StrToPtr("z"),
@@ -163,12 +164,12 @@ func TestTranslateBootDevice(t *testing.T) {
 			},
 			types.Config{
 				Ignition: types.Ignition{
-					Version: "3.2.0",
+					Version: "3.4.0",
 				},
 				Storage: types.Storage{
 					Luks: []types.Luks{
 						{
-							Clevis: &types.Clevis{
+							Clevis: types.Clevis{
 								Tang: []types.Tang{{
 									URL:        "https://example.com/",
 									Thumbprint: util.StrToPtr("z"),
@@ -177,6 +178,7 @@ func TestTranslateBootDevice(t *testing.T) {
 								Tpm2:      util.BoolToPtr(true),
 							},
 							Device:     util.StrToPtr("/dev/disk/by-partlabel/root"),
+							Discard:    util.BoolToPtr(true),
 							Label:      util.StrToPtr("luks-root"),
 							Name:       "root",
 							WipeVolume: util.BoolToPtr(true),
@@ -200,6 +202,74 @@ func TestTranslateBootDevice(t *testing.T) {
 				{From: path.New("yaml", "boot_device", "luks", "tang"), To: path.New("json", "storage", "luks", 0, "clevis", "tang")},
 				{From: path.New("yaml", "boot_device", "luks", "threshold"), To: path.New("json", "storage", "luks", 0, "clevis", "threshold")},
 				{From: path.New("yaml", "boot_device", "luks", "tpm2"), To: path.New("json", "storage", "luks", 0, "clevis", "tpm2")},
+				{From: path.New("yaml", "boot_device", "luks", "discard"), To: path.New("json", "storage", "luks", 0, "discard")},
+				{From: path.New("yaml", "boot_device", "luks"), To: path.New("json", "storage", "luks", 0, "clevis")},
+				{From: path.New("yaml", "boot_device", "luks"), To: path.New("json", "storage", "luks", 0, "device")},
+				{From: path.New("yaml", "boot_device", "luks"), To: path.New("json", "storage", "luks", 0, "label")},
+				{From: path.New("yaml", "boot_device", "luks"), To: path.New("json", "storage", "luks", 0, "name")},
+				{From: path.New("yaml", "boot_device", "luks"), To: path.New("json", "storage", "luks", 0, "wipeVolume")},
+				{From: path.New("yaml", "boot_device", "luks"), To: path.New("json", "storage", "luks", 0)},
+				{From: path.New("yaml", "boot_device", "luks"), To: path.New("json", "storage", "luks")},
+				{From: path.New("yaml", "boot_device"), To: path.New("json", "storage", "filesystems", 0, "device")},
+				{From: path.New("yaml", "boot_device"), To: path.New("json", "storage", "filesystems", 0, "format")},
+				{From: path.New("yaml", "boot_device"), To: path.New("json", "storage", "filesystems", 0, "label")},
+				{From: path.New("yaml", "boot_device"), To: path.New("json", "storage", "filesystems", 0, "wipeFilesystem")},
+				{From: path.New("yaml", "boot_device"), To: path.New("json", "storage", "filesystems", 0)},
+				{From: path.New("yaml", "boot_device"), To: path.New("json", "storage", "filesystems")},
+				{From: path.New("yaml", "boot_device"), To: path.New("json", "storage")},
+			},
+			report.Report{},
+		},
+		// LUKS, x86_64, with Tang set for offline provisioning
+		{
+			Config{
+				BootDevice: BootDevice{
+					Luks: BootDeviceLuks{
+						Tang: []base.Tang{{
+							URL:           "https://example.com/",
+							Thumbprint:    util.StrToPtr("z"),
+							Advertisement: util.StrToPtr("{\"payload\": \"xyzzy\"}"),
+						}},
+					},
+				},
+			},
+			types.Config{
+				Ignition: types.Ignition{
+					Version: "3.4.0",
+				},
+				Storage: types.Storage{
+					Luks: []types.Luks{
+						{
+							Clevis: types.Clevis{
+								Tang: []types.Tang{{
+									URL:           "https://example.com/",
+									Thumbprint:    util.StrToPtr("z"),
+									Advertisement: util.StrToPtr("{\"payload\": \"xyzzy\"}"),
+								}},
+							},
+							Device:     util.StrToPtr("/dev/disk/by-partlabel/root"),
+							Label:      util.StrToPtr("luks-root"),
+							Name:       "root",
+							WipeVolume: util.BoolToPtr(true),
+						},
+					},
+					Filesystems: []types.Filesystem{
+						{
+							Device:         "/dev/mapper/root",
+							Format:         util.StrToPtr("xfs"),
+							Label:          util.StrToPtr("root"),
+							WipeFilesystem: util.BoolToPtr(true),
+						},
+					},
+				},
+			},
+			[]translate.Translation{
+				{From: path.New("yaml", "version"), To: path.New("json", "ignition", "version")},
+				{From: path.New("yaml", "boot_device", "luks", "tang", 0, "url"), To: path.New("json", "storage", "luks", 0, "clevis", "tang", 0, "url")},
+				{From: path.New("yaml", "boot_device", "luks", "tang", 0, "thumbprint"), To: path.New("json", "storage", "luks", 0, "clevis", "tang", 0, "thumbprint")},
+				{From: path.New("yaml", "boot_device", "luks", "tang", 0, "advertisement"), To: path.New("json", "storage", "luks", 0, "clevis", "tang", 0, "advertisement")},
+				{From: path.New("yaml", "boot_device", "luks", "tang", 0), To: path.New("json", "storage", "luks", 0, "clevis", "tang", 0)},
+				{From: path.New("yaml", "boot_device", "luks", "tang"), To: path.New("json", "storage", "luks", 0, "clevis", "tang")},
 				{From: path.New("yaml", "boot_device", "luks"), To: path.New("json", "storage", "luks", 0, "clevis")},
 				{From: path.New("yaml", "boot_device", "luks"), To: path.New("json", "storage", "luks", 0, "device")},
 				{From: path.New("yaml", "boot_device", "luks"), To: path.New("json", "storage", "luks", 0, "label")},
@@ -228,7 +298,7 @@ func TestTranslateBootDevice(t *testing.T) {
 			},
 			types.Config{
 				Ignition: types.Ignition{
-					Version: "3.2.0",
+					Version: "3.4.0",
 				},
 				Storage: types.Storage{
 					Disks: []types.Disk{
@@ -309,7 +379,7 @@ func TestTranslateBootDevice(t *testing.T) {
 								"/dev/disk/by-partlabel/boot-2",
 								"/dev/disk/by-partlabel/boot-3",
 							},
-							Level:   "raid1",
+							Level:   util.StrToPtr("raid1"),
 							Name:    "md-boot",
 							Options: []types.RaidOption{"--metadata=1.0"},
 						},
@@ -319,7 +389,7 @@ func TestTranslateBootDevice(t *testing.T) {
 								"/dev/disk/by-partlabel/root-2",
 								"/dev/disk/by-partlabel/root-3",
 							},
-							Level: "raid1",
+							Level: util.StrToPtr("raid1"),
 							Name:  "md-root",
 						},
 					},
@@ -459,6 +529,7 @@ func TestTranslateBootDevice(t *testing.T) {
 			Config{
 				BootDevice: BootDevice{
 					Luks: BootDeviceLuks{
+						Discard: util.BoolToPtr(true),
 						Tang: []base.Tang{{
 							URL:        "https://example.com/",
 							Thumbprint: util.StrToPtr("z"),
@@ -473,7 +544,7 @@ func TestTranslateBootDevice(t *testing.T) {
 			},
 			types.Config{
 				Ignition: types.Ignition{
-					Version: "3.2.0",
+					Version: "3.4.0",
 				},
 				Storage: types.Storage{
 					Disks: []types.Disk{
@@ -554,7 +625,7 @@ func TestTranslateBootDevice(t *testing.T) {
 								"/dev/disk/by-partlabel/boot-2",
 								"/dev/disk/by-partlabel/boot-3",
 							},
-							Level:   "raid1",
+							Level:   util.StrToPtr("raid1"),
 							Name:    "md-boot",
 							Options: []types.RaidOption{"--metadata=1.0"},
 						},
@@ -564,13 +635,13 @@ func TestTranslateBootDevice(t *testing.T) {
 								"/dev/disk/by-partlabel/root-2",
 								"/dev/disk/by-partlabel/root-3",
 							},
-							Level: "raid1",
+							Level: util.StrToPtr("raid1"),
 							Name:  "md-root",
 						},
 					},
 					Luks: []types.Luks{
 						{
-							Clevis: &types.Clevis{
+							Clevis: types.Clevis{
 								Tang: []types.Tang{{
 									URL:        "https://example.com/",
 									Thumbprint: util.StrToPtr("z"),
@@ -579,6 +650,7 @@ func TestTranslateBootDevice(t *testing.T) {
 								Tpm2:      util.BoolToPtr(true),
 							},
 							Device:     util.StrToPtr("/dev/md/md-root"),
+							Discard:    util.BoolToPtr(true),
 							Label:      util.StrToPtr("luks-root"),
 							Name:       "root",
 							WipeVolume: util.BoolToPtr(true),
@@ -706,6 +778,7 @@ func TestTranslateBootDevice(t *testing.T) {
 				{From: path.New("yaml", "boot_device", "luks", "tang"), To: path.New("json", "storage", "luks", 0, "clevis", "tang")},
 				{From: path.New("yaml", "boot_device", "luks", "threshold"), To: path.New("json", "storage", "luks", 0, "clevis", "threshold")},
 				{From: path.New("yaml", "boot_device", "luks", "tpm2"), To: path.New("json", "storage", "luks", 0, "clevis", "tpm2")},
+				{From: path.New("yaml", "boot_device", "luks", "discard"), To: path.New("json", "storage", "luks", 0, "discard")},
 				{From: path.New("yaml", "boot_device", "luks"), To: path.New("json", "storage", "luks", 0, "clevis")},
 				{From: path.New("yaml", "boot_device", "luks"), To: path.New("json", "storage", "luks", 0, "device")},
 				{From: path.New("yaml", "boot_device", "luks"), To: path.New("json", "storage", "luks", 0, "label")},
@@ -734,6 +807,7 @@ func TestTranslateBootDevice(t *testing.T) {
 				BootDevice: BootDevice{
 					Layout: util.StrToPtr("aarch64"),
 					Luks: BootDeviceLuks{
+						Discard: util.BoolToPtr(true),
 						Tang: []base.Tang{{
 							URL:        "https://example.com/",
 							Thumbprint: util.StrToPtr("z"),
@@ -748,7 +822,7 @@ func TestTranslateBootDevice(t *testing.T) {
 			},
 			types.Config{
 				Ignition: types.Ignition{
-					Version: "3.2.0",
+					Version: "3.4.0",
 				},
 				Storage: types.Storage{
 					Disks: []types.Disk{
@@ -805,7 +879,7 @@ func TestTranslateBootDevice(t *testing.T) {
 								"/dev/disk/by-partlabel/boot-1",
 								"/dev/disk/by-partlabel/boot-2",
 							},
-							Level:   "raid1",
+							Level:   util.StrToPtr("raid1"),
 							Name:    "md-boot",
 							Options: []types.RaidOption{"--metadata=1.0"},
 						},
@@ -814,13 +888,13 @@ func TestTranslateBootDevice(t *testing.T) {
 								"/dev/disk/by-partlabel/root-1",
 								"/dev/disk/by-partlabel/root-2",
 							},
-							Level: "raid1",
+							Level: util.StrToPtr("raid1"),
 							Name:  "md-root",
 						},
 					},
 					Luks: []types.Luks{
 						{
-							Clevis: &types.Clevis{
+							Clevis: types.Clevis{
 								Tang: []types.Tang{{
 									URL:        "https://example.com/",
 									Thumbprint: util.StrToPtr("z"),
@@ -829,6 +903,7 @@ func TestTranslateBootDevice(t *testing.T) {
 								Tpm2:      util.BoolToPtr(true),
 							},
 							Device:     util.StrToPtr("/dev/md/md-root"),
+							Discard:    util.BoolToPtr(true),
 							Label:      util.StrToPtr("luks-root"),
 							Name:       "root",
 							WipeVolume: util.BoolToPtr(true),
@@ -927,6 +1002,7 @@ func TestTranslateBootDevice(t *testing.T) {
 				{From: path.New("yaml", "boot_device", "luks", "tang"), To: path.New("json", "storage", "luks", 0, "clevis", "tang")},
 				{From: path.New("yaml", "boot_device", "luks", "threshold"), To: path.New("json", "storage", "luks", 0, "clevis", "threshold")},
 				{From: path.New("yaml", "boot_device", "luks", "tpm2"), To: path.New("json", "storage", "luks", 0, "clevis", "tpm2")},
+				{From: path.New("yaml", "boot_device", "luks", "discard"), To: path.New("json", "storage", "luks", 0, "discard")},
 				{From: path.New("yaml", "boot_device", "luks"), To: path.New("json", "storage", "luks", 0, "clevis")},
 				{From: path.New("yaml", "boot_device", "luks"), To: path.New("json", "storage", "luks", 0, "device")},
 				{From: path.New("yaml", "boot_device", "luks"), To: path.New("json", "storage", "luks", 0, "label")},
@@ -955,6 +1031,7 @@ func TestTranslateBootDevice(t *testing.T) {
 				BootDevice: BootDevice{
 					Layout: util.StrToPtr("ppc64le"),
 					Luks: BootDeviceLuks{
+						Discard: util.BoolToPtr(true),
 						Tang: []base.Tang{{
 							URL:        "https://example.com/",
 							Thumbprint: util.StrToPtr("z"),
@@ -969,7 +1046,7 @@ func TestTranslateBootDevice(t *testing.T) {
 			},
 			types.Config{
 				Ignition: types.Ignition{
-					Version: "3.2.0",
+					Version: "3.4.0",
 				},
 				Storage: types.Storage{
 					Disks: []types.Disk{
@@ -1026,7 +1103,7 @@ func TestTranslateBootDevice(t *testing.T) {
 								"/dev/disk/by-partlabel/boot-1",
 								"/dev/disk/by-partlabel/boot-2",
 							},
-							Level:   "raid1",
+							Level:   util.StrToPtr("raid1"),
 							Name:    "md-boot",
 							Options: []types.RaidOption{"--metadata=1.0"},
 						},
@@ -1035,13 +1112,13 @@ func TestTranslateBootDevice(t *testing.T) {
 								"/dev/disk/by-partlabel/root-1",
 								"/dev/disk/by-partlabel/root-2",
 							},
-							Level: "raid1",
+							Level: util.StrToPtr("raid1"),
 							Name:  "md-root",
 						},
 					},
 					Luks: []types.Luks{
 						{
-							Clevis: &types.Clevis{
+							Clevis: types.Clevis{
 								Tang: []types.Tang{{
 									URL:        "https://example.com/",
 									Thumbprint: util.StrToPtr("z"),
@@ -1050,6 +1127,7 @@ func TestTranslateBootDevice(t *testing.T) {
 								Tpm2:      util.BoolToPtr(true),
 							},
 							Device:     util.StrToPtr("/dev/md/md-root"),
+							Discard:    util.BoolToPtr(true),
 							Label:      util.StrToPtr("luks-root"),
 							Name:       "root",
 							WipeVolume: util.BoolToPtr(true),
@@ -1128,6 +1206,7 @@ func TestTranslateBootDevice(t *testing.T) {
 				{From: path.New("yaml", "boot_device", "luks", "tang"), To: path.New("json", "storage", "luks", 0, "clevis", "tang")},
 				{From: path.New("yaml", "boot_device", "luks", "threshold"), To: path.New("json", "storage", "luks", 0, "clevis", "threshold")},
 				{From: path.New("yaml", "boot_device", "luks", "tpm2"), To: path.New("json", "storage", "luks", 0, "clevis", "tpm2")},
+				{From: path.New("yaml", "boot_device", "luks", "discard"), To: path.New("json", "storage", "luks", 0, "discard")},
 				{From: path.New("yaml", "boot_device", "luks"), To: path.New("json", "storage", "luks", 0, "clevis")},
 				{From: path.New("yaml", "boot_device", "luks"), To: path.New("json", "storage", "luks", 0, "device")},
 				{From: path.New("yaml", "boot_device", "luks"), To: path.New("json", "storage", "luks", 0, "label")},
@@ -1186,6 +1265,7 @@ func TestTranslateBootDevice(t *testing.T) {
 				},
 				BootDevice: BootDevice{
 					Luks: BootDeviceLuks{
+						Discard: util.BoolToPtr(true),
 						Tang: []base.Tang{{
 							URL:        "https://example.com/",
 							Thumbprint: util.StrToPtr("z"),
@@ -1200,7 +1280,7 @@ func TestTranslateBootDevice(t *testing.T) {
 			},
 			types.Config{
 				Ignition: types.Ignition{
-					Version: "3.2.0",
+					Version: "3.4.0",
 				},
 				Storage: types.Storage{
 					Disks: []types.Disk{
@@ -1259,7 +1339,7 @@ func TestTranslateBootDevice(t *testing.T) {
 								"/dev/disk/by-partlabel/boot-1",
 								"/dev/disk/by-partlabel/boot-2",
 							},
-							Level:   "raid1",
+							Level:   util.StrToPtr("raid1"),
 							Name:    "md-boot",
 							Options: []types.RaidOption{"--metadata=1.0"},
 						},
@@ -1268,13 +1348,13 @@ func TestTranslateBootDevice(t *testing.T) {
 								"/dev/disk/by-partlabel/root-1",
 								"/dev/disk/by-partlabel/root-2",
 							},
-							Level: "raid1",
+							Level: util.StrToPtr("raid1"),
 							Name:  "md-root",
 						},
 					},
 					Luks: []types.Luks{
 						{
-							Clevis: &types.Clevis{
+							Clevis: types.Clevis{
 								Tang: []types.Tang{{
 									URL:        "https://example.com/",
 									Thumbprint: util.StrToPtr("z"),
@@ -1283,6 +1363,7 @@ func TestTranslateBootDevice(t *testing.T) {
 								Tpm2:      util.BoolToPtr(true),
 							},
 							Device:     util.StrToPtr("/dev/md/md-root"),
+							Discard:    util.BoolToPtr(true),
 							Label:      util.StrToPtr("luks-root"),
 							Name:       "root",
 							WipeVolume: util.BoolToPtr(true),
@@ -1378,6 +1459,7 @@ func TestTranslateBootDevice(t *testing.T) {
 				{From: path.New("yaml", "boot_device", "luks", "tang"), To: path.New("json", "storage", "luks", 0, "clevis", "tang")},
 				{From: path.New("yaml", "boot_device", "luks", "threshold"), To: path.New("json", "storage", "luks", 0, "clevis", "threshold")},
 				{From: path.New("yaml", "boot_device", "luks", "tpm2"), To: path.New("json", "storage", "luks", 0, "clevis", "tpm2")},
+				{From: path.New("yaml", "boot_device", "luks", "discard"), To: path.New("json", "storage", "luks", 0, "discard")},
 				{From: path.New("yaml", "boot_device", "luks"), To: path.New("json", "storage", "luks", 0, "clevis")},
 				{From: path.New("yaml", "boot_device", "luks"), To: path.New("json", "storage", "luks", 0, "device")},
 				{From: path.New("yaml", "boot_device", "luks"), To: path.New("json", "storage", "luks", 0, "label")},
@@ -1413,7 +1495,139 @@ func TestTranslateBootDevice(t *testing.T) {
 
 	for i, test := range tests {
 		t.Run(fmt.Sprintf("translate %d", i), func(t *testing.T) {
-			actual, translations, r := test.in.ToIgn3_2Unvalidated(common.TranslateOptions{})
+			actual, translations, r := test.in.ToIgn3_4Unvalidated(common.TranslateOptions{})
+			r = confutil.TranslateReportPaths(r, translations)
+			baseutil.VerifyReport(t, test.in, r)
+			assert.Equal(t, test.out, actual, "translation mismatch")
+			assert.Equal(t, test.report, r, "report mismatch")
+			baseutil.VerifyTranslations(t, translations, test.exceptions)
+			assert.NoError(t, translations.DebugVerifyCoverage(actual), "incomplete TranslationSet coverage")
+		})
+	}
+}
+
+// TestTranslateGrub tests translating the Butane config Grub section.
+func TestTranslateGrub(t *testing.T) {
+	// Some tests below have the same translations
+	translations := []translate.Translation{
+		{From: path.New("yaml", "version"), To: path.New("json", "ignition", "version")},
+		{From: path.New("yaml", "grub", "users"), To: path.New("json", "storage")},
+		{From: path.New("yaml", "grub", "users"), To: path.New("json", "storage", "filesystems")},
+		{From: path.New("yaml", "grub", "users"), To: path.New("json", "storage", "filesystems", 0)},
+		{From: path.New("yaml", "grub", "users"), To: path.New("json", "storage", "filesystems", 0, "path")},
+		{From: path.New("yaml", "grub", "users"), To: path.New("json", "storage", "filesystems", 0, "device")},
+		{From: path.New("yaml", "grub", "users"), To: path.New("json", "storage", "filesystems", 0, "format")},
+		{From: path.New("yaml", "grub", "users"), To: path.New("json", "storage", "files")},
+		{From: path.New("yaml", "grub", "users"), To: path.New("json", "storage", "files", 0)},
+		{From: path.New("yaml", "grub", "users"), To: path.New("json", "storage", "files", 0, "path")},
+		{From: path.New("yaml", "grub", "users"), To: path.New("json", "storage", "files", 0, "append")},
+		{From: path.New("yaml", "grub", "users"), To: path.New("json", "storage", "files", 0, "append", 0)},
+		{From: path.New("yaml", "grub", "users"), To: path.New("json", "storage", "files", 0, "append", 0, "source")},
+		{From: path.New("yaml", "grub", "users"), To: path.New("json", "storage", "files", 0, "append", 0, "compression")},
+	}
+	tests := []struct {
+		in         Config
+		out        types.Config
+		exceptions []translate.Translation
+		report     report.Report
+	}{
+		// config with 1 user
+		{
+			Config{
+				Grub: Grub{
+					Users: []GrubUser{
+						{
+							Name:         "root",
+							PasswordHash: util.StrToPtr("grub.pbkdf2.sha512.10000.874A958E526409..."),
+						},
+					},
+				},
+			},
+			types.Config{
+				Ignition: types.Ignition{
+					Version: "3.4.0",
+				},
+				Storage: types.Storage{
+					Filesystems: []types.Filesystem{
+						{
+							Device: "/dev/disk/by-label/boot",
+							Format: util.StrToPtr("ext4"),
+							Path:   util.StrToPtr("/boot"),
+						},
+					},
+					Files: []types.File{
+						{
+							Node: types.Node{
+								Path: "/boot/grub2/user.cfg",
+							},
+							FileEmbedded1: types.FileEmbedded1{
+								Append: []types.Resource{
+									{
+										Source:      util.StrToPtr("data:,%23%20Generated%20by%20Butane%0A%0Aset%20superusers%3D%22root%22%0Apassword_pbkdf2%20root%20grub.pbkdf2.sha512.10000.874A958E526409...%0A"),
+										Compression: util.StrToPtr(""),
+									},
+								},
+							},
+						},
+					},
+				},
+			},
+			translations,
+			report.Report{},
+		},
+		// config with 2 users (and 2 different hashes)
+		{
+			Config{
+				Grub: Grub{
+					Users: []GrubUser{
+						{
+							Name:         "root1",
+							PasswordHash: util.StrToPtr("grub.pbkdf2.sha512.10000.874A958E526409..."),
+						},
+						{
+							Name:         "root2",
+							PasswordHash: util.StrToPtr("grub.pbkdf2.sha512.10000.874B829D126209..."),
+						},
+					},
+				},
+			},
+			types.Config{
+				Ignition: types.Ignition{
+					Version: "3.4.0",
+				},
+				Storage: types.Storage{
+					Filesystems: []types.Filesystem{
+						{
+							Device: "/dev/disk/by-label/boot",
+							Format: util.StrToPtr("ext4"),
+							Path:   util.StrToPtr("/boot"),
+						},
+					},
+					Files: []types.File{
+						{
+							Node: types.Node{
+								Path: "/boot/grub2/user.cfg",
+							},
+							FileEmbedded1: types.FileEmbedded1{
+								Append: []types.Resource{
+									{
+										Source:      util.StrToPtr("data:;base64,H4sIAAAAAAAC/3zMsQrCMBDG8b1PcdT9SI62JoODRfExJCGngtCEuwTx7UWyiss3fH/47eDCG0uonCC+YW01bDwMyhW0FZamLHoYJedq4bs0DiWovrKka4nPdCPo8S4tYn9QH2G2hNYYY9Dtp6Of3XmmZTIeEX8C9BdYHfmTpYU68AkAAP//Mp8bt7YAAAA="),
+										Compression: util.StrToPtr("gzip"),
+									},
+								},
+							},
+						},
+					},
+				},
+			},
+			translations,
+			report.Report{},
+		},
+	}
+
+	for i, test := range tests {
+		t.Run(fmt.Sprintf("translate %d", i), func(t *testing.T) {
+			actual, translations, r := test.in.ToIgn3_4Unvalidated(common.TranslateOptions{})
 			r = confutil.TranslateReportPaths(r, translations)
 			baseutil.VerifyReport(t, test.in, r)
 			assert.Equal(t, test.out, actual, "translation mismatch")
